@@ -1,4 +1,14 @@
-{-# LANGUAGE RecordWildCards #-}
+{-# LANGUAGE RecordWildCards, ScopedTypeVariables #-}
+
+------------------------------------------------------------------------------
+-- |
+-- Module:      Database.PostgreSQL.Simple.Transaction
+-- Copyright:   (c) 2011-2013 Leon P Smith
+--              (c) 2013 Joey Adams
+-- License:     BSD3
+-- Maintainer:  Leon P Smith <leon@melding-monads.com>
+--
+------------------------------------------------------------------------------
 
 module Database.PostgreSQL.Simple.Transaction
     (
@@ -45,7 +55,7 @@ import Database.PostgreSQL.Simple.Compat (mask, (<>))
 
 -- | Of the four isolation levels defined by the SQL standard,
 -- these are the three levels distinguished by PostgreSQL as of version 9.0.
--- See <http://www.postgresql.org/docs/9.1/static/transaction-iso.html>
+-- See <https://www.postgresql.org/docs/9.5/static/transaction-iso.html>
 -- for more information.   Note that prior to PostgreSQL 9.0, 'RepeatableRead'
 -- was equivalent to 'Serializable'.
 
@@ -132,7 +142,7 @@ withTransactionMode :: TransactionMode -> Connection -> IO a -> IO a
 withTransactionMode mode conn act =
   mask $ \restore -> do
     beginMode mode conn
-    r <- restore act `E.onException` rollback conn
+    r <- restore act `E.onException` rollback_ conn
     commit conn
     return r
 
@@ -157,7 +167,7 @@ withTransactionModeRetry mode shouldRetry conn act =
         r <- act'
         case r of
             Left e -> do
-                rollback conn
+                rollback_ conn
                 case fmap shouldRetry (E.fromException e) of
                   Just True -> retryLoop act'
                   _ -> E.throwIO e
@@ -168,6 +178,10 @@ withTransactionModeRetry mode shouldRetry conn act =
 rollback :: Connection -> IO ()
 rollback conn = execute_ conn "ABORT" >> return ()
 
+-- | Rollback a transaction, ignoring any @IOErrors@
+rollback_ :: Connection -> IO ()
+rollback_ conn = rollback conn `E.catch` \(_ :: IOError) -> return ()
+
 -- | Commit a transaction.
 commit :: Connection -> IO ()
 commit conn = execute_ conn "COMMIT" >> return ()
@@ -203,7 +217,7 @@ beginMode mode conn = do
 -- be used inside of a transaction, and provides a sort of
 -- \"nested transaction\".
 --
--- See <http://www.postgresql.org/docs/current/static/sql-savepoint.html>
+-- See <https://www.postgresql.org/docs/9.5/static/sql-savepoint.html>
 withSavepoint :: Connection -> IO a -> IO a
 withSavepoint conn body =
   mask $ \restore -> do
